@@ -48,7 +48,7 @@
 #include "runopts.h"
 #include "signkey.h"
 #include "buffer.h"
-#include "util.h"
+#include "dbutil.h"
 
 #include "genrsa.h"
 #include "gendss.h"
@@ -61,6 +61,8 @@ static void printhelp(char * progname);
 #define DSS_SIZE (1024/8) /* 1024 bit */
 
 static void buf_writefile(buffer * buf, const char * filename);
+
+/* Print a help message */
 static void printhelp(char * progname) {
 
 	fprintf(stderr, "Usage: %s -t <type> -f <filename> [-s bits]\n"
@@ -78,7 +80,12 @@ static void printhelp(char * progname) {
 					progname);
 }
 
+#if defined(DBMULTI_KEY) || !defined(DROPBEAR_MULTI)
+#if defined(DBMULTI_KEY) && defined(DROPBEAR_MULTI)
+int dropbearkey_main(int argc, char ** argv) {
+#else
 int main(int argc, char ** argv) {
+#endif
 
 	int i;
 	char ** next = 0;
@@ -135,6 +142,7 @@ int main(int argc, char ** argv) {
 #ifdef DROPBEAR_DSS
 				"dss\n"
 #endif
+				"\n"
 			   );
 		printhelp(argv[0]);
 		exit(EXIT_FAILURE);
@@ -229,7 +237,9 @@ int main(int argc, char ** argv) {
 
 	return EXIT_SUCCESS;
 }
+#endif
 
+/* Write a buffer to a file specified, failing if the file exists */
 static void buf_writefile(buffer * buf, const char * filename) {
 
 	int fd;
